@@ -72,7 +72,7 @@ nvidia_vma_open(struct vm_area_struct *vma)
 
     if (at != NULL)
     {
-        atomic64_inc(&at->usage_count);
+        NV_ATOMIC_INC(at->usage_count);
 
         NV_PRINT_AT(NV_DBG_MEMINFO, at);
     }
@@ -128,9 +128,6 @@ nvidia_vma_access(
     void *kernel_mapping;
     const nv_alloc_mapping_context_t *mmap_context = &nvlfp->mmap_context;
     NvU64 offsInVma = addr - vma->vm_start;
-    NvBool bIsNuma = NV_FALSE;
-
-    bIsNuma = pfn_valid(mmap_context->access_start >> PAGE_SHIFT);
 
     pageIndex = (offsInVma >> PAGE_SHIFT);
     pageOffset = (offsInVma & ~PAGE_MASK);
@@ -167,17 +164,10 @@ nvidia_vma_access(
         pageIndex = nv_array_index_no_speculate(pageIndex, at->num_pages);
         kernel_mapping = (void *)(at->page_table[pageIndex].virt_addr + pageOffset);
     }
-    else if (bIsNuma)
-    {
-        struct page *pPage = NV_GET_PAGE_STRUCT(mmap_context->page_array[pageIndex]);
-        NvU8 *pPagePtr = (NvU8 *) page_address(pPage);
-        kernel_mapping = &pPagePtr[pageOffset];
-    }
     else
     {
         NvU64 idx = 0;
         NvU64 curOffs = 0;
-        
         for(; idx < mmap_context->memArea.numRanges; idx++)
         {
             NvU64 nextOffs = mmap_context->memArea.pRanges[idx].size + curOffs;
@@ -200,19 +190,12 @@ found:
 
     length = NV_MIN(length, (int)(PAGE_SIZE - pageOffset));
 
-#if defined(NVCPU_AARCH64)
-    if (write)
-        memcpy_toio(kernel_mapping, buffer, length);
-    else
-        memcpy_fromio(buffer, kernel_mapping, length);
-#else
     if (write)
         memcpy(kernel_mapping, buffer, length);
     else
         memcpy(buffer, kernel_mapping, length);
-#endif // defined(NVCPU_AARCH64)
 
-    if (at == NULL && !bIsNuma)
+    if (at == NULL)
     {
         kernel_mapping = ((char *)kernel_mapping - pageOffset);
         os_unmap_kernel_space(kernel_mapping, PAGE_SIZE);
@@ -351,15 +334,21 @@ int nv_encode_caching(
                     NV_PGPROT_UNCACHED(*prot) :
                     NV_PGPROT_UNCACHED_DEVICE(*prot);
             break;
-#if defined(NV_PGPROT_WRITE_COMBINED) && \
-    defined(NV_PGPROT_WRITE_COMBINED_DEVICE)
+#if defined(NV_PGPROT_WRITE_COMBINED)
         case NV_MEMORY_DEFAULT:
         case NV_MEMORY_WRITECOMBINED:
             if (NV_ALLOW_WRITE_COMBINING(memory_type))
             {
+#if defined(NVCPU_RISCV64)
+                /* 
+                 * Don't attempt to mark sysmem pages as write combined on riscv.
+                 * Bug 5404055 to clean up this check.
+                 */
                 *prot = (memory_type == NV_MEMORY_TYPE_FRAMEBUFFER) ?
-                        NV_PGPROT_WRITE_COMBINED_DEVICE(*prot) :
-                        NV_PGPROT_WRITE_COMBINED(*prot);
+                    NV_PGPROT_WRITE_COMBINED(*prot) : *prot;
+#else
+                *prot = NV_PGPROT_WRITE_COMBINED(*prot);
+#endif
                 break;
             }
 
@@ -425,7 +414,7 @@ static int nvidia_mmap_sysmem(
     int ret = 0;
     unsigned long start = 0;
 
-    atomic64_inc(&at->usage_count);
+    NV_ATOMIC_INC(at->usage_count);
 
     start = vma->vm_start;
     for (j = page_index; j < (page_index + pages); j++)
@@ -461,7 +450,7 @@ static int nvidia_mmap_sysmem(
 
         if (ret)
         {
-            atomic64_dec(&at->usage_count);
+            NV_ATOMIC_DEC(at->usage_count);
             nv_printf(NV_DBG_ERRORS,
                       "NVRM: Userspace mapping creation failed [%d]!\n", ret);
             return -EAGAIN;
@@ -607,7 +596,8 @@ int nvidia_mmap_helper(
             // TODO: Refactor is needed as part of bug#2001704.
             //
             if ((nv_get_numa_status(nvl) == NV_NUMA_STATUS_ONLINE) &&
-                !IS_REG_OFFSET(nv, access_start, access_len) &&
+                pfn_to_page(__phys_to_pfn(access_start)) != NULL &&
+                pfn_to_page(__phys_to_pfn(access_start + access_len - 1)) != NULL &&
                 (mmap_context->num_pages != 0))
             {
                 ret = nvidia_mmap_numa(vma, mmap_context);
@@ -911,7 +901,7 @@ static NvBool nv_vma_enter_locked(struct vm_area_struct *vma, NvBool detaching)
     {
         // Clean up on error: release refcount and dep_map
         refcount_sub_and_test(VMA_LOCK_OFFSET, &vma->vm_refcnt);
-        rwsem_release(&vma->vmlock_dep_map, _RET_IP_);
+        rwsem_release(&vma->vmlock_dep_map, _RET_IP_);	
         return NV_FALSE;
     }
 
