@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1999-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -110,21 +110,13 @@ nv_procfs_read_gpu_info(
     char *type;
     const char *name;
     char *uuid;
-    char vbios_version[15];
     nvidia_stack_t *sp = NULL;
-    char firmware_version[64] = { 0 };
 
     if (nv_kmem_cache_alloc_stack(&sp) != 0)
     {
         return 0;
     }
 
-    if (rm_ref_dynamic_power(sp, nv, NV_DYNAMIC_PM_COARSE) != NV_OK)
-    {
-        nv_kmem_cache_free_stack(sp);
-        return 0;
-    }
-
     name = rm_get_device_name(pci_dev->device,
                               pci_dev->subsystem_vendor,
                               pci_dev->subsystem_device);
@@ -132,18 +124,23 @@ nv_procfs_read_gpu_info(
     seq_printf(s, "Model: \t\t %s\n", name);
     seq_printf(s, "IRQ:   \t\t %d\n", nv->interrupt_line);
 
-    uuid = rm_get_gpu_uuid(sp, nv);
-
-    if (uuid != NULL)
+    if (nv->nv_uuid_cache.valid)
     {
-        seq_printf(s, "GPU UUID: \t %s\n", uuid);
-        os_free_mem(uuid);
-        uuid = NULL;
+        uuid = rm_get_gpu_uuid(sp, nv);
+
+        if (uuid != NULL)
+        {
+            seq_printf(s, "GPU UUID: \t %s\n", uuid);
+            os_free_mem(uuid);
+            uuid = NULL;
+        }
+    }
+    else
+    {
+        nv_printf(NV_DBG_ERRORS, "GPU UUID cache not valid!\n");
     }
 
-    rm_get_vbios_version(sp, nv, vbios_version);
-
-    seq_printf(s, "Video BIOS: \t %s\n", vbios_version);
+    seq_printf(s, "Video BIOS: \t %s\n", nv->cached_gpu_info.vbios_version);
 
     if (nv_find_pci_capability(pci_dev, PCI_CAP_ID_EXP))
         type = "PCIe";
@@ -159,10 +156,13 @@ nv_procfs_read_gpu_info(
                    nv->pci_info.slot, PCI_FUNC(pci_dev->devfn));
     seq_printf(s, "Device Minor: \t %u\n", nvl->minor_num);
 
-    rm_get_firmware_version(sp, nv, firmware_version, sizeof(firmware_version));
-    if (firmware_version[0] != '\0')
+    if (nv->cached_gpu_info.firmware_version[0] == '\0')
     {
-        seq_printf(s, "GPU Firmware: \t %s\n", firmware_version);
+        seq_printf(s, "GPU Firmware: \t N/A\n");
+    }
+    else
+    {
+        seq_printf(s, "GPU Firmware: \t %s\n", nv->cached_gpu_info.firmware_version);
     }
 
 #if defined(DEBUG)
@@ -180,7 +180,6 @@ nv_procfs_read_gpu_info(
     seq_printf(s, "GPU Excluded:\t %s\n",
                ((nv->flags & NV_FLAG_EXCLUDE) != 0) ? "Yes" : "No");
 
-    rm_unref_dynamic_power(sp, nv, NV_DYNAMIC_PM_COARSE);
 
     nv_kmem_cache_free_stack(sp);
 
@@ -206,6 +205,8 @@ nv_procfs_read_power(
 
     rm_get_power_info(sp, nv, &power_info);
     seq_printf(s, "Runtime D3 status:          %s\n", power_info.dynamic_power_status);
+    seq_printf(s, "Tegra iGPU Rail-Gating:     %s\n",
+                    nv->is_tegra_pci_igpu_rg_enabled ? "Enabled" : "Disabled");
     seq_printf(s, "Video Memory:               %s\n\n", power_info.vidmem_power_status);
 
     seq_printf(s, "GPU Hardware Support:\n");
@@ -889,7 +890,7 @@ nv_procfs_close_unbind_lock(
         down(&nvl->ldata_lock);
         if ((value == 1) && !(nv->flags & NV_FLAG_UNBIND_LOCK))
         {
-            if (atomic64_read(&nvl->usage_count) == 0)
+            if (NV_ATOMIC_READ(nvl->usage_count) == 0)
                 rm_unbind_lock(sp, nv);
 
             if (nv->flags & NV_FLAG_UNBIND_LOCK)
