@@ -2,9 +2,10 @@
 #define __NV_UNIX_VERSION_H__
 
 #if defined(NV_LINUX) || defined(NV_BSD) || defined(NV_SUNOS) || defined(NV_VMWARE) || defined(NV_QNX) || defined(NV_INTEGRITY) || \
-    (defined(RMCFG_FEATURE_PLATFORM_GSP) && RMCFG_FEATURE_PLATFORM_GSP == 1)
+    (defined(RMCFG_FEATURE_PLATFORM_GSP) && RMCFG_FEATURE_PLATFORM_GSP == 1) || \
+    defined(NV_DCECORE)
 
-#define NV_VERSION_STRING               "580.126.18"
+#define NV_VERSION_STRING               "590.44.01"
 
 #else
 
