@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -27,7 +27,7 @@
  */
 
 /* ------------------------ Includes ---------------------------------------- */
-#include "objtmr.h"
+#include "gpu/timer/objtmr.h"
 #include "class/cl0004.h" // NV004_NOTIFIERS_SET_ALARM_NOTIFY
 #include "gpu/gpu_resource.h"
 #include "core/locks.h"
@@ -35,12 +35,12 @@
 #include "kernel/gpu/intr/intr.h"
 
 /* ------------------------ Static Function Prototypes ---------------------- */
-static PTMR_EVENT_PVT   _tmrPullCallbackFromHead (OBJTMR *);
+static TMR_EVENT_PVT *  _tmrPullCallbackFromHead (OBJTMR *);
 static void             _tmrScanCallback(OBJTMR *, void *);
-static void             _tmrScanCallbackOSTimer(OBJTMR *, PTMR_EVENT_PVT);
-static PTMR_EVENT_PVT   _tmrGetNextFreeCallback(OBJTMR *);
-static NV_STATUS        _tmrInsertCallback(OBJTMR *, PTMR_EVENT_PVT, NvU64);
-static void             _tmrInsertCallbackInList(OBJGPU *pGpu, OBJTMR *pTmr, PTMR_EVENT_PVT pEvent);
+static void             _tmrScanCallbackOSTimer(OBJTMR *, TMR_EVENT_PVT *);
+static TMR_EVENT_PVT *  _tmrGetNextFreeCallback(OBJTMR *);
+static NV_STATUS        _tmrInsertCallback(OBJTMR *, TMR_EVENT_PVT *, NvU64);
+static void             _tmrInsertCallbackInList(OBJGPU *pGpu, OBJTMR *pTmr, TMR_EVENT_PVT *pEvent);
 static void             _tmrStateLoadCallbacks(OBJGPU *, OBJTMR *);
 static NV_STATUS        _tmrGetNextAlarmTime(OBJTMR *, NvU64 *);
 static void             _tmrScheduleCallbackInterrupt(OBJGPU *, OBJTMR *, NvU64);
@@ -53,9 +53,6 @@ tmrConstructEngine_IMPL
     ENGDESCRIPTOR engDesc
 )
 {
-    // Mark that this timer is not yet initialized
-    pTmr->bInitialized = NV_FALSE;
-
     // Create the Granular lock for SWRL Timer callback
     pTmr->pTmrSwrlLock = portSyncSpinlockCreate(portMemAllocatorGetGlobalNonPaged());
     if (pTmr->pTmrSwrlLock == NULL)
@@ -206,7 +203,7 @@ static NV_INLINE NvBool tmrEventsExist(OBJTMR *pTmr)
     return pTmr->pRmActiveEventList != NULL;
 }
 
-static NV_INLINE NvBool tmrIsOSTimer(OBJTMR *pTmr, PTMR_EVENT pEventPublic)
+static NV_INLINE NvBool tmrIsOSTimer(OBJTMR *pTmr, TMR_EVENT *pEventPublic)
 {
     return ((pEventPublic != NULL) &&
            (pTmr->getProperty(pTmr, PDB_PROP_TMR_USE_OS_TIMER_FOR_CALLBACKS) ||
@@ -221,13 +218,13 @@ static NV_INLINE NvBool tmrIsOSTimer(OBJTMR *pTmr, PTMR_EVENT pEventPublic)
 NV_STATUS tmrEventCreate_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT *ppEventPublic,
+    TMR_EVENT **ppEventPublic,
     TIMEPROC    Proc,
     void       *pUserData,
     NvU32       flags
 )
 {
-    PTMR_EVENT_PVT *ppEvent = (PTMR_EVENT_PVT*)ppEventPublic;
+    TMR_EVENT_PVT **ppEvent = (TMR_EVENT_PVT **)ppEventPublic;
     NV_STATUS status = NV_OK;
 
     *ppEvent = portMemAllocNonPaged(sizeof(TMR_EVENT_PVT));
@@ -273,10 +270,19 @@ _tmrScheduleCallbackInterrupt
     if (pTmr->getProperty(pTmr, PDB_PROP_TMR_USE_COUNTDOWN_TIMER_FOR_RM_CALLBACKS))
     {
         NvU64 currentTime;
-        NvU32 countdownTime;
+        // 
+        // Bug: 5071665, 4417666. High spikes of DPC Activity in ARM based system.
+        // The issue is caused due to an ARM compiler optimization bug which is doing 
+        // 32-bit subtraction instead of 64-bit subtraction casted to 32-bits. 
+        // Fix: Set 'volatile' qualifier to' countdownTime' so that it is not optimized.
+        //
+        volatile NvU32 countdownTime = 0;
 
         tmrGetCurrentTime(pTmr, &currentTime);
-        countdownTime = currentTime < alarmTime ? NvU64_LO32(alarmTime - currentTime) : 0;
+        if (currentTime < alarmTime)
+        {
+            countdownTime = NvU64_LO32(alarmTime - currentTime);
+        }
         tmrSetCountdown_HAL(pGpu, pTmr, countdownTime, 0, NULL);
     }
     else
@@ -328,13 +334,13 @@ tmrGetCallbackInterruptPending_IMPL
 void tmrEventCancel_IMPL
 (
     OBJTMR         *pTmr,
-    PTMR_EVENT      pEventPublic
+    TMR_EVENT      *pEventPublic
 )
 {
     NvU64 nextAlarmTime;
     OBJGPU *pGpu = ENG_GET_GPU(pTmr);
-    PTMR_EVENT_PVT pEvent = (PTMR_EVENT_PVT)pEventPublic;
-    PTMR_EVENT_PVT pChaser = pTmr->pRmActiveEventList;
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT *)pEventPublic;
+    TMR_EVENT_PVT *pChaser = pTmr->pRmActiveEventList;
     NvBool bRemovedHead = pChaser == pEvent;
 
     if (pEventPublic == NULL)
@@ -397,10 +403,10 @@ void tmrEventCancel_IMPL
 void tmrEventDestroy_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT  pEventPublic
+    TMR_EVENT  *pEventPublic
 )
 {
-    PTMR_EVENT_PVT pEvent = (PTMR_EVENT_PVT)pEventPublic;
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT *)pEventPublic;
 
     if (pEvent != NULL)
     {
@@ -421,12 +427,51 @@ void tmrEventDestroy_IMPL
 }
 
 /*!
- * TODO: document
+ * Returns time until next callback for a given event
+ *
+ * @param[in]   pEvent   The event whose remaining time needs to be determined.
+ */
+NV_STATUS
+tmrEventTimeUntilNextCallback_IMPL
+(
+    OBJTMR     *pTmr,
+    TMR_EVENT  *pEventPublic,
+    NvU64      *pTimeUntilCallbackNs
+)
+{
+    NvU64 currentTime;
+    NvU64 nextAlarmTime;
+
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT*)pEventPublic;
+
+    if (tmrIsOSTimer(pTmr, pEventPublic))
+    {
+        currentTime = osGetMonotonicTimeNs();
+        // timens corresponds to relative time for OS timer
+        NV_CHECK_OR_RETURN(LEVEL_ERROR, portSafeAddU64(pEvent->timens, pEvent->startTimeNs, &nextAlarmTime),
+                           NV_ERR_INVALID_ARGUMENT);
+    }
+    else
+    {
+        NV_ASSERT_OK_OR_RETURN(tmrGetCurrentTime(pTmr, &currentTime));
+        // timens corresponds to abs time in case of ptimer
+        nextAlarmTime = pEvent->timens;
+    }
+    if (currentTime > nextAlarmTime)
+        return NV_ERR_INVALID_STATE;
+
+    *pTimeUntilCallbackNs = nextAlarmTime - currentTime;
+    return NV_OK;
+}
+
+
+/*!
+ * Callback invoked by NV0004_CTRL_CMD_TMR_SET_ALARM_NOTIFY
  */
 static NV_STATUS
-_nv0004CtrlCmdTmrSetAlarmNotifyCallback(OBJGPU *pGpu, OBJTMR *pTmr, void *pData)
+_nv0004CtrlCmdTmrSetAlarmNotifyCallback(OBJGPU *pGpu, OBJTMR *pTmr, TMR_EVENT *pTmrEvent)
 {
-    PEVENTNOTIFICATION pNotifyEvent = pData;
+    PEVENTNOTIFICATION pNotifyEvent = pTmrEvent->pUserData;
     NV_STATUS status = NV_OK;
 
     // perform a direct callback to the client
@@ -434,15 +479,16 @@ _nv0004CtrlCmdTmrSetAlarmNotifyCallback(OBJGPU *pGpu, OBJTMR *pTmr, void *pData)
     {
         //one shot signal
         status = osNotifyEvent(pGpu, pNotifyEvent, NV004_NOTIFIERS_SET_ALARM_NOTIFY, 0, NV_OK);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "failed to notify event in callback, status: 0x%08x\n", status);
+            return status;
+        }
     }
 
     return status;
 }
 
-/*!
- * TODO: document
- * TODO: Migrate this to match current API (probably)
- */
 NV_STATUS
 tmrapiCtrlCmdTmrSetAlarmNotify_IMPL
 (
@@ -450,8 +496,10 @@ tmrapiCtrlCmdTmrSetAlarmNotify_IMPL
     NV0004_CTRL_TMR_SET_ALARM_NOTIFY_PARAMS *pParams
 )
 {
+    NV_STATUS status;
     OBJGPU *pGpu = GPU_RES_GET_GPU(pTimerApi);
     OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
+    TMR_EVENT *pTmrEvent = NULL;
     PEVENTNOTIFICATION pNotifyEvent = inotifyGetNotificationList(staticCast(pTimerApi, INotifier));
 
     // Validate the timer event
@@ -466,11 +514,29 @@ tmrapiCtrlCmdTmrSetAlarmNotify_IMPL
         return NV_ERR_INVALID_ARGUMENT;
     }
 
-    // schedule the timer
-    tmrScheduleCallbackRel(pTmr,
-                           _nv0004CtrlCmdTmrSetAlarmNotifyCallback,
-                           pNotifyEvent,
-                           pParams->alarmTimeNsecs, 0, 0);
+    // Create an OS timer if not already created
+    if (pNotifyEvent->pTmrEvent == NULL)
+    {
+        status = tmrEventCreate(pTmr,
+                                &pTmrEvent,
+                                _nv0004CtrlCmdTmrSetAlarmNotifyCallback,
+                                (void*)pNotifyEvent,
+                                0);
+        if (status != NV_OK)
+            return status;
+
+        pNotifyEvent->pGpu = pGpu;
+        pNotifyEvent->pTmrEvent = pTmrEvent;
+    }
+    else
+    {
+        pTmrEvent = pNotifyEvent->pTmrEvent;
+    }
+
+    // Schedule the timer
+    status = tmrEventScheduleRel(pTmr, pTmrEvent, pParams->alarmTimeNsecs);
+    if (status != NV_OK)
+        return status;
 
     return NV_OK;
 }
@@ -549,7 +615,7 @@ NV_STATUS tmrGetCurrentDiffTime_IMPL
 NV_STATUS tmrEventScheduleRel_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT  pEvent,
+    TMR_EVENT  *pEvent,
     NvU64       RelTime
 )
 {
@@ -566,7 +632,7 @@ NV_STATUS tmrEventScheduleRel_IMPL
         // Capture system time here, this will help in scheduling callbacks
         // if there is a state unload before receiving the OS timer callback.
         //
-        osGetCurrentTick(&pEventPvt->startTimeNs);
+        pEventPvt->startTimeNs = osGetMonotonicTimeNs();
         if (!tmrEventOnList(pTmr, pEvent))
         {
             _tmrInsertCallback(pTmr, pEventPvt, RelTime);
@@ -640,11 +706,11 @@ NV_STATUS tmrScheduleCallbackRelSec_IMPL
 NvBool tmrEventOnList_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT  pEventPublic
+    TMR_EVENT  *pEventPublic
 )
 {
-    PTMR_EVENT_PVT pEvent = (PTMR_EVENT_PVT)pEventPublic;
-    PTMR_EVENT_PVT pScan  = tmrIsOSTimer(pTmr, pEventPublic) ?
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT *)pEventPublic;
+    TMR_EVENT_PVT *pScan  = tmrIsOSTimer(pTmr, pEventPublic) ?
                             pTmr->pRmActiveOSTimerEventList :
                             pTmr->pRmActiveEventList;
 
@@ -673,8 +739,8 @@ NvBool tmrCallbackOnList_IMPL
 )
 {
     NvBool onList = NV_FALSE;
-    PTMR_EVENT_PVT tmrScan;
-    PTMR_EVENT_PVT tmrList;
+    TMR_EVENT_PVT *tmrScan;
+    TMR_EVENT_PVT *tmrList;
 
     tmrList = pTmr->pRmActiveEventList;
 
@@ -694,13 +760,13 @@ NvBool tmrCallbackOnList_IMPL
 /*!
  * OBSOLETE: This will be removed very soon!
  */
-static PTMR_EVENT_PVT
+static TMR_EVENT_PVT *
 _tmrGetNextFreeCallback
 (
     OBJTMR *pTmr
 )
 {
-    PTMR_EVENT_PVT pEvent = NULL;
+    TMR_EVENT_PVT *pEvent = NULL;
 
     pEvent = pTmr->pRmCallbackFreeList_OBSOLETE;
     if (pEvent != NULL)
@@ -718,7 +784,7 @@ _tmrGetNextFreeCallback
  * Creates and inserts a node into the callback list.
  *
  *  @param[in]  pEvent     Callback memory structure, provided by user.
- *  @param[in]  Time       Absolute nanoseconds at which to call Proc.
+ *  @param[in]  Time       Absolute(for ptimer) or relative (for OS timer) nanoseconds at which to call Proc.
  *
  *  @returns               Status
  */
@@ -726,7 +792,7 @@ static NV_STATUS
 _tmrInsertCallback
 (
     OBJTMR         *pTmr,
-    PTMR_EVENT_PVT  pEvent,
+    TMR_EVENT_PVT *pEvent,
     NvU64           Time
 )
 {
@@ -734,7 +800,7 @@ _tmrInsertCallback
     OBJGPU *pGpu = ENG_GET_GPU(pTmr);
 
     // If this is a free callback
-    if (!pEvent->bInUse && !tmrEventOnList(pTmr, (PTMR_EVENT)pEvent))
+    if (!pEvent->bInUse && !tmrEventOnList(pTmr, (TMR_EVENT *)pEvent))
     {
         pEvent->timens    = Time;
 
@@ -765,17 +831,17 @@ _tmrInsertCallbackInList
 (
     OBJGPU         *pGpu,
     OBJTMR         *pTmr,
-    PTMR_EVENT_PVT  pEvent
+    TMR_EVENT_PVT *pEvent
 )
 {
-    PTMR_EVENT_PVT   pScan;
+    TMR_EVENT_PVT   *pScan;
     NvBool           bAddedAsHead  = NV_TRUE;
     NvU64            nextAlarmTime;
 
     NV_ASSERT(!pEvent->bInUse);
     pEvent->bInUse = NV_TRUE;
 
-    if (tmrIsOSTimer(pTmr, (PTMR_EVENT)pEvent))
+    if (tmrIsOSTimer(pTmr, (TMR_EVENT *)pEvent))
     {
         pEvent->pNext = pTmr->pRmActiveOSTimerEventList;
         pTmr->pRmActiveOSTimerEventList = pEvent;
@@ -857,7 +923,7 @@ _tmrInsertCallbackInList
 NV_STATUS tmrEventScheduleAbs_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT  pEventPublic,
+    TMR_EVENT  *pEventPublic,
     NvU64       timeAbsNs
 )
 {
@@ -866,13 +932,19 @@ NV_STATUS tmrEventScheduleAbs_IMPL
 
     if ((pEvent != NULL) && tmrIsOSTimer(pTmr, pEventPublic))
     {
+        NvU64 timeRelNs, currentTime;
+
+        rmStatus = tmrGetCurrentTime(pTmr, &currentTime);
+        if (rmStatus != NV_OK)
+            return rmStatus;
+
         //
-        // OS-Timer is supported only in Relative mode. This assert is to trap
-        // if someone trying to call tmrEventScheduleAbs for OSTimer
+        // If absolute time is less than current time, then timer has already
+        // expired. For this case, schedule timer with zero delay to trigger
+        // immediate callback.
         //
-        NV_ASSERT_FAILED("Attempting to schedule OS-Timer callback with Abs time.");
-        rmStatus = NV_ERR_INVALID_ARGUMENT;
-        return rmStatus;
+        timeRelNs = (timeAbsNs > currentTime) ? (timeAbsNs - currentTime) : 0;
+        return tmrEventScheduleRel(pTmr, pEventPublic, timeRelNs);
     }
 
     if ((pEvent == NULL) || (pEventPublic->pTimeProc == NULL &&
@@ -927,7 +999,7 @@ NV_STATUS tmrScheduleCallbackAbs_IMPL
     NvU32               ChId
 )
 {
-    PTMR_EVENT_PVT tmrInsert;
+    TMR_EVENT_PVT *tmrInsert;
     // Get a free callback from the free list.
     if(pTmr->pRmCallbackFreeList_OBSOLETE == NULL)
     {
@@ -943,7 +1015,7 @@ NV_STATUS tmrScheduleCallbackAbs_IMPL
             tmrInsert->super.flags = Flags;
             tmrInsert->super.chId = ChId;
 
-            return tmrEventScheduleAbs(pTmr, (PTMR_EVENT)tmrInsert, Time);
+            return tmrEventScheduleAbs(pTmr, (TMR_EVENT *)tmrInsert, Time);
         }
         else
         {
@@ -959,7 +1031,7 @@ NV_STATUS tmrScheduleCallbackAbs_IMPL
 }
 
 /*!
- *  Searches specified lists for PTMR_EVENT  associated with Object and
+ *  Searches specified lists for TMR_EVENT* associated with Object and
  *  removes it.
  *
  *  @param[in]  Object  Unique identifier based on TMR_POBJECT_BASE (tmr.h)
@@ -972,9 +1044,9 @@ static void _tmrScanCallback
     void   *pObject
 )
 {
-    PTMR_EVENT_PVT  tmrScan;
-    PTMR_EVENT_PVT  tmrNext;
-    PTMR_EVENT_PVT  tmrCurrent;
+    TMR_EVENT_PVT *tmrScan;
+    TMR_EVENT_PVT *tmrNext;
+    TMR_EVENT_PVT *tmrCurrent;
 
     //
     // Start at the beginning of the callback list.
@@ -1062,10 +1134,10 @@ static void
 _tmrScanCallbackOSTimer
 (
     OBJTMR *pTmr,
-    PTMR_EVENT_PVT  pEvent
+    TMR_EVENT_PVT *pEvent
 )
 {
-    PTMR_EVENT_PVT  pCurrent = pTmr->pRmActiveOSTimerEventList;
+    TMR_EVENT_PVT *pCurrent = pTmr->pRmActiveOSTimerEventList;
 
     if (pCurrent == pEvent)
     {
@@ -1111,12 +1183,12 @@ _tmrGetNextAlarmTime
  * Return the very next callback to be scheduled, removing it from the list
  * and marking it as free (only "In Use" when in the list)
  */
-static PTMR_EVENT_PVT  _tmrPullCallbackFromHead
+static TMR_EVENT_PVT * _tmrPullCallbackFromHead
 (
     OBJTMR         *pTmr
 )
 {
-    PTMR_EVENT_PVT tmrDelete = pTmr->pRmActiveEventList;
+    TMR_EVENT_PVT *tmrDelete = pTmr->pRmActiveEventList;
     if (tmrDelete)
     {
         // remove from callbackList
@@ -1180,7 +1252,7 @@ tmrCallExpiredCallbacks_IMPL
 {
     NvU64           currentTime = 0;
     NvU64           nextAlarmTime;
-    PTMR_EVENT_PVT  pEvent;
+    TMR_EVENT_PVT  *pEvent;
     NV_STATUS       rmStatus;
     NvBool          bProccessedCallback = NV_FALSE;
 
@@ -1205,34 +1277,27 @@ tmrCallExpiredCallbacks_IMPL
             // Pull from head of list.
             pEvent = _tmrPullCallbackFromHead(pTmr);
 
-            if (pEvent &&
-                ((pEvent->super.pTimeProc != NULL) ||
-                (pEvent->pTimeProc_OBSOLETE != NULL)) )
+            if (pEvent != NULL)
             {
                 // Call callback.  This could insert a new callback into the list.
-                if (pEvent->bLegacy)
+                if (pEvent->bLegacy && pEvent->pTimeProc_OBSOLETE != NULL)
                 {
                     pEvent->pTimeProc_OBSOLETE(pGpu, pTmr, pEvent->super.pUserData);
+                    bProccessedCallback = NV_TRUE;
+                }
+                else if (!pEvent->bLegacy && pEvent->super.pTimeProc != NULL)
+                {
+                    pEvent->super.pTimeProc(pGpu, pTmr, (TMR_EVENT *)pEvent);
+                    bProccessedCallback = NV_TRUE;
                 }
                 else
                 {
-                    pEvent->super.pTimeProc(pGpu, pTmr, (PTMR_EVENT)pEvent);
+                    NV_ASSERT_FAILED("Attempting to execute callback with NULL procedure.");
                 }
-
-                bProccessedCallback = NV_TRUE;
             }
             else
             {
-                //
-                // Bug 372159: Hopefully by checking that the callback procedure
-                // is not NULL in tmrEventScheduleAbs() we should never hit
-                // this point, but this is just to be certain.  If you hit this
-                // assert please update Bug 3721259 with pertinent details
-                // (Swak, !stacks, what you were developing/testing, etc.).
-                //
-                NV_ASSERT_FAILED(
-                    "Attempting to execute callback with NULL procedure.  "
-                    "Please update Bug 372159 with appropriate information.");
+                NV_ASSERT_FAILED("Attempting to execute callback with NULL timer event.");
             }
         }
 
@@ -1264,7 +1329,7 @@ _tmrStateLoadCallbacks
 )
 {
     NvU64 nextAlarmTime = 0;
-    PTMR_EVENT_PVT pScan = pTmr->pRmActiveOSTimerEventList;
+    TMR_EVENT_PVT *pScan = pTmr->pRmActiveOSTimerEventList;
 
     if (tmrEventsExist(pTmr))
     {
@@ -1297,8 +1362,8 @@ _tmrStateLoadCallbacks
         // Capture system time here, this will help in scheduling callbacks
         // if there is a state unload before receiving the OS timer callback.
         //
-        osGetCurrentTick(&pScan->startTimeNs);
-        tmrEventScheduleRelOSTimer_HAL(pTmr, (PTMR_EVENT)pScan, pScan->timens);
+        pScan->startTimeNs = osGetMonotonicTimeNs();
+        tmrEventScheduleRelOSTimer_HAL(pTmr, (TMR_EVENT *)pScan, pScan->timens);
         pScan = pScan->pNext;
     }
 }
@@ -1377,7 +1442,7 @@ tmrGetSystemTime_IMPL
     if (pTime != NULL)
     {
         // Get the system time and calculate the contents of the returned structure.
-        osGetCurrentTime(&sec, &usec);
+        osGetSystemTime(&sec, &usec);
         pTime->days = sec / (3600 * 24);            // # of days since ref point
         sec = sec % (3600 * 24);                    // seconds since day began
         pTime->msecs = sec * 1000 + (usec / 1000);  // milliseconds since day began
@@ -1394,7 +1459,7 @@ tmrCheckCallbacksReleaseSem_IMPL
     NvU32   chId
 )
 {
-    PTMR_EVENT_PVT pScan;
+    TMR_EVENT_PVT *pScan;
 
     for (pScan = pTmr->pRmActiveEventList; pScan != NULL; pScan = pScan->pNext)
     {
@@ -1524,9 +1589,17 @@ tmrDiffExceedsTime_IMPL
     return bRetVal;
 }
 
-/*!
- * TODO: document
- */
+NV_STATUS
+tmrStatePreInitLocked_IMPL
+(
+    OBJGPU *pGpu,
+    OBJTMR *pTmr
+)
+{
+
+    return NV_OK;
+}
+
 NV_STATUS
 tmrStateInitLocked_IMPL
 (
@@ -1566,7 +1639,7 @@ tmrStateUnload_IMPL
     NvU32   flags
 )
 {
-    PTMR_EVENT_PVT pScan = pTmr->pRmActiveOSTimerEventList;
+    TMR_EVENT_PVT *pScan = pTmr->pRmActiveOSTimerEventList;
     NvU64 currentSysTime, elapsedTime;
 
     // Disable Timer interrupt.
@@ -1579,7 +1652,7 @@ tmrStateUnload_IMPL
     //
     while (pScan != NULL)
     {
-        osGetCurrentTick(&currentSysTime);
+        currentSysTime = osGetMonotonicTimeNs();
         //
         // If somehow any of the time difference is negative,
         // we will use the  original time duration.
@@ -1593,7 +1666,7 @@ tmrStateUnload_IMPL
             }
         }
 
-        tmrEventCancelOSTimer_HAL(pTmr, (PTMR_EVENT)pScan);
+        tmrEventCancelOSTimer_HAL(pTmr, (TMR_EVENT *)pScan);
         pScan = pScan->pNext;
     }
     return NV_OK;
@@ -1667,7 +1740,17 @@ tmrapiDeregisterEvents_IMPL(TimerApi *pTimerApi)
     // Validate the timer event
     while (pNotifyEvent != NULL)
     {
-        tmrCancelCallback(pTmr, pNotifyEvent);
+        //
+        // TimerApi events are only set through NV0004_CTRL_CMD_TMR_SET_ALARM_NOTIFY
+        // which only schedules TMR_EVENT type callbacks. So only call the new API.
+        //
+        TMR_EVENT *pTmrEvent = pNotifyEvent->pTmrEvent;
+        if (pTmrEvent != NULL)
+        {
+            tmrEventDestroy(pTmr, pTmrEvent);
+            pNotifyEvent->pGpu = NULL;
+            pNotifyEvent->pTmrEvent = NULL;
+        }
 
         pNotifyEvent = pNotifyEvent->Next;
     }
@@ -1698,7 +1781,7 @@ static NV_STATUS _tmrCallbackWrapperfunction
 (
     OBJGPU *pGpu,
     OBJTMR *pTmr,
-    PTMR_EVENT pEvent
+    TMR_EVENT *pEvent
 )
 {
     wrapperStorage_t *pObj_Inner = (wrapperStorage_t *)pEvent->pUserData;
@@ -1739,7 +1822,7 @@ tmrCtrlCmdEventCreate
 )
 {
     NV_STATUS         rc;
-    PTMR_EVENT        pEvent;
+    TMR_EVENT        *pEvent;
     wrapperStorage_t *pWrapper;
     OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
 
@@ -1767,7 +1850,6 @@ tmrCtrlCmdEventCreate
 /*!
  * Schedules an existing event. Takes in time arguments and a flag to
  * determine if it should be interpreted as absolute or relative time.
- * While using OSTimer, use this api only with relative time.
  *
  * @returns NV_STATUS
  */
@@ -1780,19 +1862,10 @@ tmrCtrlCmdEventSchedule
 {
     NV_STATUS rc;
     OBJTMR    *pTmr   = GPU_GET_TIMER(pGpu);
-    TMR_EVENT *pEvent = (PTMR_EVENT)NvP64_VALUE(pParams->pEvent);
+    TMR_EVENT *pEvent = (TMR_EVENT *)NvP64_VALUE(pParams->pEvent);
 
     if (pParams->bUseTimeAbs)
     {
-        //
-        // FIXME: This function is called only from dynamic_power.c file,
-        // using OSTimer. And OSTimer is always scheduled in relative mode
-        //
-        if ((pEvent != NULL) && tmrIsOSTimer(pTmr, pEvent))
-        {
-            NV_ASSERT_FAILED("Attempting to schedule OSTimer with Abs time.");
-            return NV_ERR_INVALID_ARGUMENT;
-        }
         rc = tmrEventScheduleAbs(pTmr, pEvent, pParams->timeNs);
     }
     else
@@ -1816,7 +1889,7 @@ tmrCtrlCmdEventCancel
 )
 {
     OBJTMR    *pTmr   = GPU_GET_TIMER(pGpu);
-    PTMR_EVENT pEvent = (PTMR_EVENT)NvP64_VALUE(pParams->pEvent);
+    TMR_EVENT *pEvent = (TMR_EVENT *)NvP64_VALUE(pParams->pEvent);
     tmrEventCancel(pTmr, pEvent);
 
     return NV_OK;
@@ -1836,7 +1909,7 @@ tmrCtrlCmdEventDestroy
 )
 {
     OBJTMR    *pTmr   = GPU_GET_TIMER(pGpu);
-    PTMR_EVENT pEvent = (PTMR_EVENT)NvP64_VALUE(pParams->pEvent);
+    TMR_EVENT *pEvent = (TMR_EVENT *)NvP64_VALUE(pParams->pEvent);
 
     // Free our temporary wrapper storage
     portMemFree(pEvent->pUserData);
@@ -1850,18 +1923,18 @@ NV_STATUS tmrEventServiceTimer_IMPL
 (
     OBJGPU             *pGpu,
     OBJTMR             *pTmr,
-    PTMR_EVENT          pPublicEvent
+    TMR_EVENT          *pPublicEvent
 )
 {
-    PTMR_EVENT_PVT pEvent = (PTMR_EVENT_PVT)pPublicEvent;
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT *)pPublicEvent;
     NV_STATUS status      = NV_ERR_INVALID_REQUEST;
 
-    if ((pEvent == NULL) || !tmrIsOSTimer(pTmr, (PTMR_EVENT)pEvent))
+    if ((pEvent == NULL) || !tmrIsOSTimer(pTmr, (TMR_EVENT *)pEvent))
     {
         return status;
     }
 
-    if (tmrEventOnList(pTmr, (PTMR_EVENT)pEvent))
+    if (tmrEventOnList(pTmr, (TMR_EVENT *)pEvent))
     {
         _tmrScanCallbackOSTimer(pTmr, pEvent);
         status = tmrEventServiceOSTimerCallback_HAL(pGpu, pTmr, pPublicEvent);
