@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -30,7 +30,6 @@
 #include "gpu/subdevice/generic_engine.h"
 #include "gpu/subdevice/subdevice.h"
 #include "gpu/mem_mgr/mem_mgr.h"
-#include "mem_mgr/fla_mem.h"
 
 #include "class/cl0000.h" // NV01_NULL_OBJECT
 
@@ -102,56 +101,7 @@ rmapiGetEffectiveAddrSpace
     NV_ADDRESS_SPACE *pAddrSpace
 )
 {
-    NV_ADDRESS_SPACE addrSpace;
-    NvBool bDirectSysMappingAllowed = NV_TRUE;
-
-    KernelBus *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
-
-    NV_ASSERT_OK_OR_RETURN(
-        kbusIsDirectMappingAllowed_HAL(pGpu, pKernelBus, pMemDesc, mapFlags,
-                                      &bDirectSysMappingAllowed));
-
-    //
-    // Bug 1482818: Deprecate reflected mappings in production code.
-    //  The usage of reflected writes, in addition to causing several deadlock
-    //  scenarios involving P2P transfers, are disallowed on NVLINK (along with
-    //  reflected reads), and should no longer be used.
-    //  The below PDB property should be unset once the remaining usages in MODS
-    //  have been culled. (Bug 1780557)
-    //
-    if ((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) &&
-        !bDirectSysMappingAllowed &&
-        (DRF_VAL(OS33, _FLAGS, _MAPPING, mapFlags) != NVOS33_FLAGS_MAPPING_DIRECT) &&
-        !kbusIsReflectedMappingAccessAllowed(pKernelBus))
-    {
-        NV_ASSERT(0);
-        return NV_ERR_NOT_SUPPORTED;
-    }
-
-    if (memdescGetFlag(pMemDesc, MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1))
-    {
-        addrSpace = ADDR_FBMEM;
-    }
-    else if ((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) &&
-        (bDirectSysMappingAllowed || FLD_TEST_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, mapFlags) ||
-        (IS_VIRTUAL_WITH_SRIOV(pGpu) && !IS_FMODEL(pGpu) && !IS_RTLSIM(pGpu))))
-    {
-        addrSpace = ADDR_SYSMEM;
-    }
-    else if ((memdescGetAddressSpace(pMemDesc) == ADDR_FBMEM) ||
-             ((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) && !bDirectSysMappingAllowed))
-    {
-        addrSpace = ADDR_FBMEM;
-    }
-    else
-    {
-        addrSpace = memdescGetAddressSpace(pMemDesc);
-    }
-
-    if (pAddrSpace)
-        *pAddrSpace = addrSpace;
-
-    return NV_OK;
+    return kbusGetEffectiveAddressSpace_HAL(pGpu, pMemDesc, mapFlags, pAddrSpace);
 }
 
 // Asserts to check caching type matches across sdk and nv_memory_types
@@ -216,11 +166,27 @@ memMap_IMPL
     NV_ASSERT_OR_RETURN(pMemoryInfo != NULL, NV_ERR_NOT_SUPPORTED);
     pMemDesc = pMemoryInfo->pMemDesc;
 
-    if ((pMemoryInfo->categoryClassId == NV01_MEMORY_SYSTEM_OS_DESCRIPTOR) &&
-        !(memdescGetAddressSpace(pMemDesc) == ADDR_FBMEM &&
-          RMCFG_FEATURE_PLATFORM_MODS))
+    if (pMemoryInfo->categoryClassId == NV01_MEMORY_SYSTEM_OS_DESCRIPTOR)
     {
-        return NV_ERR_NOT_SUPPORTED;
+        NvBool isSupported = NV_FALSE;
+        NV_ADDRESS_SPACE addressSpace = memdescGetAddressSpace(pMemDesc);
+
+        if (memdescGetFlag(pMemDesc, MEMDESC_FLAGS_ALLOW_EXT_SYSMEM_USER_CPU_MAPPING))
+        {
+            isSupported = NV_TRUE;
+        }
+
+        if ((addressSpace == ADDR_FBMEM) && RMCFG_FEATURE_MODS_FEATURES)
+        {
+            isSupported = NV_TRUE;
+        }
+
+        if (!isSupported)
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                "CPU mapping not supported for addressSpace: 0x%x\n", addressSpace);
+            return NV_ERR_NOT_SUPPORTED;
+        }
     }
 
     //
@@ -242,17 +208,27 @@ memMap_IMPL
         }
     }
 
-    if ((pGpu != NULL) && (pMemoryInfo->Flags & NVOS32_ALLOC_FLAGS_PROTECTED))
+    if ((pGpu != NULL) && gpuIsCCFeatureEnabled(pGpu) &&
+        (pMemoryInfo->Flags & NVOS32_ALLOC_FLAGS_PROTECTED))
     {
         ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
         //
         // If neither BAR1 nor PCIE as a whole is trusted, fail the mapping
         // for allocations in CPR region. Mapping should still succeed for
         // allocations in non-CPR region
+        // Deny BAR1 access to CPU-RM by default irrespective of prod or devtools
+        // mode. Some mappings made by CPU-RM may be allowed to go thorough in
+        // devtools mode.
+        // However, allow the mapping to go through on platforms where GSP-DMA
+        // is not present e.g. MODS. User may have also set a regkey to force
+        // BAR accesses.
         //
-        if ((pCC != NULL) && !pCC->ccStaticInfo.bIsBar1Trusted &&
-            !pCC->ccStaticInfo.bIsPcieTrusted)
+        if (((pCC != NULL) && !pCC->ccStaticInfo.bIsBar1Trusted &&
+            !pCC->ccStaticInfo.bIsPcieTrusted) ||
+            (IS_GSP_CLIENT(pGpu) && pMapParams->bKernel && !pKernelBus->bForceBarAccessOnHcc &&
+             FLD_TEST_DRF(OS33, _FLAGS, _ALLOW_MAPPING_ON_HCC, _NO, pMapParams->flags)))
         {
+            NV_PRINTF(LEVEL_ERROR, "BAR1 mapping to CPR vidmem not supported\n");
             NV_ASSERT(0);
             return NV_ERR_NOT_SUPPORTED;
         }
@@ -300,13 +276,17 @@ memMap_IMPL
         effectiveAddrSpace = ADDR_SYSMEM;
     }
 
-    bIsSysmem = (effectiveAddrSpace == ADDR_SYSMEM);
-    bIsSysmem = bIsSysmem || (effectiveAddrSpace == ADDR_EGM);
+    bIsSysmem = (effectiveAddrSpace == ADDR_SYSMEM) || (effectiveAddrSpace == ADDR_EGM);
 
-    if (dynamicCast(pMemoryInfo, FlaMemory) != NULL)
+    //
+    // MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1 indicates a special mapping type of HW registers,
+    // so map it as device memory (uncached).
+    //
+    NvU32 cachingType = NV_MEMORY_WRITECOMBINED;
+    if (pMemDesc != NULL && !memdescHasSubDeviceMemDescs(pMemDesc))
     {
-        NV_PRINTF(LEVEL_WARNING, "CPU mapping to FLA memory not allowed\n");
-        return NV_ERR_NOT_SUPPORTED;
+        cachingType = memdescGetFlag(pMemDesc, MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1) ?
+                      NV_MEMORY_UNCACHED : NV_MEMORY_WRITECOMBINED;
     }
 
     //
@@ -323,38 +303,26 @@ memMap_IMPL
         {
             if (pMapParams->bKernel)
             {
-                if (pMemDesc->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS)
-                {
-                    NvP64 tempCpuPtr = kbusMapCoherentCpuMapping_HAL(pGpu, pKernelBus, pMemDesc);
-                    if (tempCpuPtr == NULL)
-                    {
-                        rmStatus = NV_ERR_GENERIC;
-                    }
-                    else
-                    {
-                        rmStatus = NV_OK;
-                        tempCpuPtr = NvP64_PLUS_OFFSET(tempCpuPtr, pMapParams->offset);
-                    }
-                    *pMapParams->ppCpuVirtAddr = tempCpuPtr;
+                rmStatus = kbusMapCoherentCpuMapping_HAL(pGpu,
+                                                         pKernelBus,
+                                                         pMemDesc,
+                                                         pMapParams->offset,
+                                                         pMapParams->length,
+                                                         pMapParams->protect,
+                                                         pMapParams->ppCpuVirtAddr,
+                                                         &priv);
 
-                    if (rmStatus != NV_OK)
-                        return rmStatus;
-                }
-                else
-                {
-                    rmStatus = osMapSystemMemory(pMemDesc,
-                                                 pMapParams->offset,
-                                                 pMapParams->length,
-                                                 pMapParams->bKernel,
-                                                 pMapParams->protect,
-                                                 pMapParams->ppCpuVirtAddr,
-                                                 &priv);
-                    if (rmStatus != NV_OK)
-                        return rmStatus;
-                }
+                // No memarea support for kernel yet
+
+                if (rmStatus != NV_OK)
+                    return rmStatus;
             }
             else
             {
+                NvU64               paddr;
+                NvBool              bContigDesc;
+                ADDRESS_TRANSLATION addressTranslation = AT_CPU;
+                NvU64               pageSize;
 
                 //
                 // Allocating mapping for user mode client
@@ -364,22 +332,80 @@ memMap_IMPL
                 //       this function will transition to storing the mapping parameters onto
                 //       the FD.  Also note: All mapping parameters are ignored (!).
                 //
-                //   For now, we're going to return the first page of the nvlink aperture
-                //   mapping of this allocation.  See nvidia_mmap_helper for establishment
-                //   of direct mapping.
-                //
+                //   TODO: refactor this to either use the new osMapMemoryArea interface or unify with kernel in osMapSystemMemory
 
-                rmStatus = osMapPciMemoryUser(pGpu->pOsGpuInfo,
-                                              ((NvUPtr)pKernelMemorySystem->coherentCpuFbBase +
-                                               (NvUPtr)memdescGetPhysAddr(pMemDesc,
-                                                AT_CPU, pMapParams->offset)),
-                                              pMapParams->length,
-                                              pMapParams->protect,
-                                              pMapParams->ppCpuVirtAddr,
-                                              &priv,
-                                              NV_MEMORY_UNCACHED);
-                if (rmStatus != NV_OK)
-                    return rmStatus;
+                paddr = ((NvU64) pKernelMemorySystem->coherentCpuFbBase) +
+                    ((NvU64) memdescGetPhysAddr(pMemDesc, AT_CPU, pMapParams->offset));
+                *((NvU64*) pMapParams->ppCpuVirtAddr) = paddr;
+                bContigDesc = memdescGetContiguity(pMemDesc, addressTranslation);
+                pageSize = memdescGetPageSize(pMemDesc, addressTranslation);
+
+                //
+                // For FB addresses onlined to kernel, we will go through the NUMA APIs and don't need
+                // to store off more map info.
+                // For FB addresses not onlined to kernel(RM reserved memory region or CDMM mode),
+                // fill in the os map information so we can map it later (including the memArea so we
+                // can map it disconig)
+                //
+                if (!osNumaOnliningEnabled(pGpu->pOsGpuInfo) ||
+                    !IS_COHERENT_CPU_ATS_OFFSET(pGpu, pKernelMemorySystem,
+                                                paddr,
+                                                bContigDesc ? pMapParams->length :
+                                                NV_MIN(pMapParams->length,
+                                                       NV_ALIGN_UP(pMapParams->offset + 1, pageSize) -
+                                                       pMapParams->offset)))
+                {
+                    MemoryArea memArea;
+                    NvU64    i;
+                    NvU64    mapGranularity;
+                    NvU64    offset;
+                    NvU64    mapRangeEndPlus1;
+                    NvU64    numRanges = 0;
+                    MemoryRange mapRange = mrangeMake(pMapParams->offset, pMapParams->length);
+
+                    mapRangeEndPlus1 = mapRange.start + mapRange.size;
+                    // TODO: simplify for dynamic page granularity
+                    mapGranularity = bContigDesc ? mapRange.size : pageSize;
+
+                    NV_CHECK_OR_RETURN(LEVEL_INFO,
+                                        mapRangeEndPlus1 <= memdescGetSize(pMemDesc),
+                                        NV_ERR_OUT_OF_RANGE);
+
+                    for (offset = mapRange.start; offset < mapRangeEndPlus1; numRanges++)
+                    {
+                        offset = bContigDesc ? (offset + mapGranularity) : NV_ALIGN_DOWN(offset + mapGranularity, mapGranularity);
+                    }
+
+                    memArea.pRanges = portMemAllocNonPaged(sizeof(MemoryRange) * numRanges);
+                    NV_CHECK_OR_RETURN(LEVEL_INFO, memArea.pRanges != NULL, NV_ERR_NO_MEMORY);
+
+                    memArea.numRanges = numRanges;
+
+                    for (i = 0, offset = mapRange.start; offset < mapRangeEndPlus1; i++)
+                    {
+                        memArea.pRanges[i].start = memdescGetPhysAddr(pMemDesc, addressTranslation, offset) + pKernelMemorySystem->coherentCpuFbBase;
+                        memArea.pRanges[i].size  = bContigDesc ? mapGranularity : NV_MIN(mapRangeEndPlus1, NV_ALIGN_UP(offset + 1, mapGranularity)) - offset;
+
+                        offset = bContigDesc ? (offset + mapGranularity) : NV_ALIGN_DOWN(offset + mapGranularity, mapGranularity);
+                    }
+
+                    // memArea.pRanges[0].start is the same as what is stored in the pMapParams->ppCpuVirtAddr above
+
+                    rmStatus = osMapPciMemoryAreaUser(pGpu->pOsGpuInfo,
+                                                      memArea,
+                                                      pMapParams->protect,
+                                                      cachingType,
+                                                      pMapParams->ppCpuVirtAddr,
+                                                      &priv);
+
+                    // Coherent path doesn't need the memArea stored off for unmap
+                    portMemFree(memArea.pRanges);
+
+                    if (rmStatus != NV_OK)
+                        return rmStatus;
+                }
+
+                rmStatus = NV_OK;
             }
 
             NV_PRINTF(LEVEL_INFO,
@@ -394,8 +420,8 @@ memMap_IMPL
                                                     priv,
                                                     *(pMapParams->ppCpuVirtAddr),
                                                     pMapParams->length,
-                                                    -1,
-                                                    -1,
+                                                    0,
+                                                    0,
                                                     pMapParams->flags);
             pCpuMapping->pPrivate->pGpu = pGpu;
 
@@ -420,17 +446,8 @@ memMap_IMPL
         NvBool bcState = NV_FALSE;
         NvU64 gpuVirtAddr = 0;
         NvU64 gpuMapLength = 0;
-
-        //
-        // MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1 indicates a special mapping type of HW registers,
-        // so map it as device memory (uncached).
-        //
-        NvU32 cachingType = NV_MEMORY_WRITECOMBINED;
-        if (pMemDesc != NULL && !memdescHasSubDeviceMemDescs(pMemDesc))
-        {
-            cachingType = memdescGetFlag(pMemDesc, MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1) ?
-                          NV_MEMORY_UNCACHED : NV_MEMORY_WRITECOMBINED;
-        }
+        MemoryArea memArea;
+        NvBool bUseMemArea = NV_FALSE;
 
         if (!kbusIsBar1PhysicalModeEnabled(pKernelBus))
         {
@@ -464,6 +481,9 @@ memMap_IMPL
 
                 // Below, we only map one GPU's address for CPU access, so we can use UNICAST here
                 NvU32 busMapFbFlags = BUS_MAP_FB_FLAGS_MAP_UNICAST;
+#if defined(NV_UNIX)
+                busMapFbFlags |= pMapParams->bKernel ? 0 : BUS_MAP_FB_FLAGS_ALLOW_DISCONTIG;
+#endif
                 if(DRF_VAL(OS33, _FLAGS, _MAPPING, pMapParams->flags) == NVOS33_FLAGS_MAPPING_DIRECT)
                 {
                     busMapFbFlags |= BUS_MAP_FB_FLAGS_DISABLE_ENCRYPTION;
@@ -482,7 +502,7 @@ memMap_IMPL
                 pMemDesc = memdescGetMemDescFromGpu(pMemDesc, pGpu);
 
                 // WAR for Bug 3564398, need to allocate doorbell for windows differently
-                if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM &&
+                if (RMCFG_FEATURE_PLATFORM_WINDOWS &&
                     memdescGetFlag(pMemDesc, MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1))
                 {
                     busMapFbFlags |= BUS_MAP_FB_FLAGS_MAP_DOWNWARDS;
@@ -490,11 +510,17 @@ memMap_IMPL
 
                 (void) deviceGetByHandle(staticCast(pClient, RsClient),
                                          pMapParams->hDevice, &pDevice);
-
+                bUseMemArea = NV_TRUE;
                 rmStatus = kbusMapFbAperture_HAL(pGpu, pKernelBus,
-                                                 pMemDesc, pMapParams->offset,
-                                                 &gpuVirtAddr, &gpuMapLength,
-                                                 busMapFbFlags, pDevice);
+                                                   pMemDesc, mrangeMake(pMapParams->offset, gpuMapLength),
+                                                   &memArea, busMapFbFlags, pDevice);
+
+                if (rmStatus == NV_OK)
+                {
+                    gpuVirtAddr = memArea.pRanges[0].start;
+                }
+
+
             }
 
             if (rmStatus != NV_OK)
@@ -519,7 +545,7 @@ memMap_IMPL
                                               pMapParams->ppCpuVirtAddr,
                                               cachingType);
         }
-        else
+        else if(!bUseMemArea)
         {
             rmStatus = osMapPciMemoryUser(pGpu->pOsGpuInfo,
                                           (kbusIsBar1PhysicalModeEnabled(pKernelBus)?
@@ -530,6 +556,29 @@ memMap_IMPL
                                           &priv,
                                           cachingType);
         }
+        else
+        {
+            NvU64 idx;
+            NvU64 barAddr = gpumgrGetGpuPhysFbAddr(pGpu);
+
+            for (idx = 0; idx < memArea.numRanges; idx++)
+            {
+                memArea.pRanges[idx].start += barAddr;
+            }
+
+            rmStatus = osMapPciMemoryAreaUser(pGpu->pOsGpuInfo,
+                                          memArea,
+                                          pMapParams->protect,
+                                          cachingType,
+                                          pMapParams->ppCpuVirtAddr,
+                                          &priv);
+
+            for (idx = 0; idx < memArea.numRanges; idx++)
+            {
+                memArea.pRanges[idx].start -= barAddr;
+            }
+
+        }
 
         //
         // It's possible that NVOS33_FLAGS_MAPPING is set to NVOS33_FLAGS_MAPPING_DIRECT
@@ -548,12 +597,17 @@ memMap_IMPL
                                                 *(pMapParams->ppCpuVirtAddr),
                                                 pMapParams->length,
                                                 kbusIsBar1PhysicalModeEnabled(pKernelBus)
-                                                    ? (NvU64)-1
+                                                    ? (NvU64)0
                                                     : gpuVirtAddr,
                                                 kbusIsBar1PhysicalModeEnabled(pKernelBus)
-                                                    ? (NvU64)-1
+                                                    ? (NvU64)0
                                                     : gpuMapLength,
                                                 pMapParams->flags);
+
+        if (bUseMemArea)
+        {
+            pCpuMapping->pPrivate->memArea = memArea;
+        }
         pCpuMapping->pPrivate->pGpu = pGpu;
 
         if (rmStatus != NV_OK)
@@ -567,11 +621,10 @@ memMap_IMPL
             if (!kbusIsBar1PhysicalModeEnabled(pKernelBus))
             {
                 kbusUnmapFbAperture_HAL(pGpu,
-                                        pKernelBus,
-                                        pMemDesc,
-                                        gpuVirtAddr,
-                                        gpuMapLength,
-                                        BUS_MAP_FB_FLAGS_MAP_UNICAST);
+                                          pKernelBus,
+                                          pMemDesc,
+                                          memArea,
+                                          BUS_MAP_FB_FLAGS_MAP_UNICAST);
     _rmMapMemory_busFail:
                 gpumgrSetBcEnabledStatus(pGpu, bcState);
             }
@@ -641,8 +694,8 @@ memMap_IMPL
                                                 priv,
                                                 *(pMapParams->ppCpuVirtAddr),
                                                 pMapParams->length,
-                                                -1, // gpu virtual addr
-                                                -1, // gpu map length
+                                                0, // gpu virtual addr
+                                                0, // gpu map length
                                                 pMapParams->flags);
         pCpuMapping->pPrivate->pGpu = pGpu;
 
@@ -707,18 +760,19 @@ memUnmap_IMPL
 
         if (pCpuMapping->pPrivate->bKernel)
         {
-            if(pMemDesc->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS)
+            kbusUnmapCoherentCpuMapping_HAL(pGpu, pKernelBus, pMemDesc,
+                                            pCpuMapping->pLinearAddress,
+                                            pCpuMapping->pPrivate->pPriv);
+        }
+        else
+        {
+            if (!osNumaOnliningEnabled(pGpu->pOsGpuInfo))
             {
-                NV_ASSERT(pMemDesc->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS);
-                kbusUnmapCoherentCpuMapping_HAL(pGpu, pKernelBus, pMemDesc);
-            }
-            else
-            {
-                osUnmapSystemMemory(pMemDesc,
-                                    pCpuMapping->pPrivate->bKernel,
-                                    pCpuMapping->processId,
-                                    pCpuMapping->pLinearAddress,
-                                    pCpuMapping->pPrivate->pPriv);
+                RmUnmapBusAperture(pGpu,
+                       pCpuMapping->pLinearAddress,
+                       pCpuMapping->length,
+                       pCpuMapping->pPrivate->bKernel,
+                       pCpuMapping->pPrivate->pPriv);
             }
         }
 
@@ -733,15 +787,14 @@ memUnmap_IMPL
         //
     }
     // System Memory case
-    else if ((pGpu == NULL) || (((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM)
-                                 || (memdescGetAddressSpace(pMemDesc) == ADDR_EGM)
-                                ) && FLD_TEST_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, pCpuMapping->flags)))
+    else if ((pGpu == NULL) || (((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) ||
+                                  (memdescGetAddressSpace(pMemDesc) == ADDR_EGM)) &&
+                                 FLD_TEST_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, pCpuMapping->flags)))
     {
         if (FLD_TEST_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, pCpuMapping->flags))
         {
             memdescUnmap(pMemDesc,
                          pCpuMapping->pPrivate->bKernel,
-                         pCpuMapping->processId,
                          pCpuMapping->pLinearAddress,
                          pCpuMapping->pPrivate->pPriv);
         }
@@ -759,36 +812,18 @@ memUnmap_IMPL
         if (!kbusIsBar1PhysicalModeEnabled(pKernelBus))
         {
             {
+                MEMORY_DESCRIPTOR *pMemDesc = memdescGetMemDescFromGpu(pMemory->pMemDesc, pGpu);
+
                 kbusUnmapFbAperture_HAL(pGpu, pKernelBus,
-                                        pMemory->pMemDesc,
-                                        pCpuMapping->pPrivate->gpuAddress,
-                                        pCpuMapping->pPrivate->gpuMapLength,
-                                        BUS_MAP_FB_FLAGS_MAP_UNICAST);
+                                          pMemDesc,
+                                          pCpuMapping->pPrivate->memArea,
+                                          BUS_MAP_FB_FLAGS_MAP_UNICAST);
             }
         }
     }
     else if (memdescGetAddressSpace(pMemDesc) == ADDR_VIRTUAL)
     {
-        // If the memory is tiled, then it's being mapped through BAR1
-        if( DRF_VAL(OS32, _ATTR, _TILED, pMemory->Attr) )
-        {
-            // BAR1 mapping.  Unmap it.
-            if (pCpuMapping->pPrivate->bKernel)
-            {
-                osUnmapPciMemoryKernel64(pGpu, pCpuMapping->pLinearAddress);
-            }
-            else
-            {
-                osUnmapPciMemoryUser(pGpu->pOsGpuInfo,
-                                     pCpuMapping->pLinearAddress,
-                                     pCpuMapping->length,
-                                     pCpuMapping->pPrivate->pPriv);
-            }
-        }
-        else
-        {
-            NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_STATE);
-        }
+        NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_STATE);
     }
     else if (memdescGetAddressSpace(pMemDesc) == ADDR_REGMEM)
     {
@@ -1158,7 +1193,23 @@ rmapiMapToCpuWithSecInfoV2
     NV_PRINTF(LEVEL_INFO, "MMU_PROFILER Nv04MapMemory 0x%x\n", *flags);
 
     portMemSet(&lockInfo, 0, sizeof(lockInfo));
-    rmapiInitLockInfo(pRmApi, hClient, &lockInfo);
+    status = rmapiInitLockInfo(pRmApi, hClient, NV01_NULL_OBJECT, &lockInfo);
+    if (status != NV_OK)
+    {
+        rmapiEpilogue(pRmApi, &rmApiContext);
+        return status;
+    }
+
+    //
+    // In the RTD3 case, the API lock isn't taken since it can be initiated
+    // from another thread that holds the API lock and because we now hold
+    // the GPU lock.
+    //
+    if (rmapiInRtd3PmPath())
+    {
+        lockInfo.flags |= RM_LOCK_FLAGS_NO_API_LOCK;
+        lockInfo.state &= ~RM_LOCK_STATES_API_LOCK_ACQUIRED;
+    }
 
     LOCK_METER_DATA(MAPMEM, flags, 0, 0);
 
@@ -1328,7 +1379,12 @@ rmapiUnmapFromCpuWithSecInfo
         return status;
 
     portMemSet(&lockInfo, 0, sizeof(lockInfo));
-    rmapiInitLockInfo(pRmApi, hClient, &lockInfo);
+    status = rmapiInitLockInfo(pRmApi, hClient, NV01_NULL_OBJECT, &lockInfo);
+    if (status != NV_OK)
+    {
+        rmapiEpilogue(pRmApi, &rmApiContext);
+        return NV_OK;
+    }
 
     LOCK_METER_DATA(UNMAPMEM, flags, 0, 0);
 
